{
  started 1/1/98.
}
unit bhChDate;

interface

uses
  Wintypes, winprocs, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Mask, bhDate, myDate;

type
  TbhChDate = class(TWinControl)
  private
    theDate: TbhDate;
    dayName: TMaskEdit;
    IncBtn : TButton;
    DecBtn : TButton;

    dayStr : string;


    FUseDayName : boolean;
    FDay : DayFormat;

    function CreateDate : TbhDate;
    function CreateButton(cap : string) : TButton;
    function CreateDayName : TMaskEdit;

    procedure DoIncDec(Sender : TObject);

    procedure SetRequiredDayName(Value : boolean);
    procedure SetDayFormat(Value : DayFormat);

    procedure CalculateDay;

    procedure CMExit(var Msg: TCMExit); message CM_EXIT;

  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;

    function SetDateByString(const Value : string) : boolean;
    procedure SetDateByDate(const Value : TDateTime);
    procedure SetDateByJulianDate(const Value : JulianDate);

    function GetDay : string;
    
    function GetDateAsString : string;
    function GetDateAsDate : TDateTime;
    function GetDateAsJulianDate : JulianDate;

  published
    property RequiredDayName : boolean
       read FUseDayName write SetRequiredDayName default False;
       
    property FormatOfDayName : DayFormat
       read FDay write SetDayFormat default ShortDay;
  end;

procedure Register;

implementation
uses myString;

{
  1/1/98.

  Layout the pieces of the component.
}
constructor TbhChDate.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);

  FUseDayName := False;

  dayStr := '';
  FDay := ShortDay;

  {IncBtn is positioned where the user choose to drop the component at.}
  IncBtn := CreateButton('+');

  DecBtn := CreateButton('-');
{  DecBtn is positioned immediately below IncBtn.}
  DecBtn.Top := IncBtn.Top + IncBtn.Height + 2;

{  ...and so on.}
  theDate := CreateDate;
  theDate.Left := DecBtn.Width + 2;
  theDate.Top := IncBtn.Top + 1;

  self.Width := 100;
  self.Height := 25;
end;

destructor TbhChDate.Destroy;
begin
  inherited Destroy;
end;


function TbhChDate.CreateDate : TbhDate;
begin
  Result := TbhDate.Create(self);         {Result must be used.}
  result.width := 100;
  Result.Parent := self;                  {Owned by this contaner.}
  Result.Visible := True;                 {Must be visible.}
end;

{
  3/1/98.

  Create a TButton object, set its properties in the context of this
  component.
}
function TbhChDate.CreateButton(cap : string) : TButton;
begin
  Result := TButton.Create(self);
  Result.Parent := self;
  Result.Visible := True;
  Result.OnClick := DoIncDec;
  Result.Height := 11;
  Result.Width := 14;
  Result.TabStop := False;
  Result.Caption := cap;
end;

function TbhChDate.CreateDayName : TMaskEdit;
begin
  Result := TMaskEdit.Create(self);
  Result.Parent := self;
  Result.Visible := True;
end;

{
  1/1/98.
  Set increment, decrement.
}
procedure TbhChDate.DoIncDec(Sender : TObject);
var
  dStr : DDMMYYYY;
begin
  if (not self.theDate.DateIsValid(self.theDate.Text)) then
    exit;

  dStr := self.theDate.GetDateAsString;

  if (Sender = DecBtn) then
    myDate .IncDecFromDateStr(-1, dStr, dayStr)
  else
    myDate .IncDecFromDateStr(1, dStr, dayStr);

  self.theDate.SetDateByString(dStr);

  if (FUseDayName) then
    self.dayName.Text := dayStr;
end;

{
  3/1/98.
}
procedure TbhChDate.SetRequiredDayName(Value : boolean);
var
  prevVal : boolean;
begin
  prevVal := FUseDayName;

  if (Value <> FUseDayName) then
    begin {//Value <> FUseDayName}

      FUseDayName := Value;

{ If the dayName has previously been created,
free it (realease memory); then set width of the container accordingly.}
      if (prevVal) then
        begin
          dayName.Free;
          self.Width := 160;
        end
      else
{      //If dayName has not been created, create and put it in the appropriate
      //position. Set the width of the container accordingly.}
        begin {//not prevVal}
          dayName := CreateDayName;
          dayName.Width := 67;
          dayName.Height := theDate.Height;
          dayName.Top := theDate.Top;
          dayName.Left := theDate.Left + theDate.Width + 2;
          dayName.Enabled := false;

          self.Width := 160;
        end;{  //not prevVal}

    end;  {//Value <> FUseDayName}
end;

{
  4/1/97.
}
procedure TbhChDate.SetDayFormat(Value : DayFormat);
begin
  if (Value <> FDay) then
    FDay := Value;
end;

{
  4/1/98.
}
procedure TbhChDate.CalculateDay;
begin
  if (self.theDate.DateIsValid(self.theDate.Text)) then
    if (dayStr = '') then
      begin
        dayStr := myDate .DayOfWeekFromDateStr(self.theDate.Text, FDay);

        if (FUseDayName) then
          self.dayName.Text := dayStr;
      end;
end;

{
  4/1/98.
}
procedure TbhChDate.CMExit(var Msg: TCMExit);
begin
  CalculateDay;
end;

{
   5/1/98.
}
function TbhChDate.SetDateByString(const Value : string) : boolean;
begin
  SetDateByString := true;

 { //If the date is valid then calculate the day name.}
  if (self.theDate.SetDateByString(Value)) then
    CalculateDay
{  //If the date is not valid then clear the day name as well as clearing
  //the day name text box if one is used.}
  else
    begin
      SetDateByString := false;
      dayStr := '';
      if (FUseDayName) then
        self.dayName.Text := dayStr;
    end;
end;

{
   5/1/98.
}
procedure TbhChDate.SetDateByDate(const Value : TDateTime);
begin
  self.theDate.SetDateByDate(Value);
end;

{
   5/1/98.
}
procedure TbhChDate.SetDateByJulianDate(const Value : JulianDate);
begin
  self.theDate.SetDateByJulianDate(Value);
end;
  
{
  4/1/98.
}
function TbhChDate.GetDay : string;
begin
  CalculateDay;
  GetDay := dayStr;
end;

{
   5/1/98.
}
function TbhChDate.GetDateAsString : string;
var
  s : string;
begin
  GetDateAsString := '';

  if (self.theDate.DateIsValid(self.theDate.Text)) then
    begin
      s := Remove_Embeded_Spaces(self.theDate.Text);
      GetDateAsString := s;
    end;

end;

{
   5/1/98.
}
function TbhChDate.GetDateAsDate : TDateTime;
var
  s : string;
begin
  GetDateAsDate := 0;

  if (self.theDate.DateIsValid(self.theDate.Text)) then
    begin
      s := Remove_Embeded_Spaces(self.theDate.Text);
      GetDateAsDate := StrToDate(s);
    end;
end;

{
   5/1/98.
}
function TbhChDate.GetDateAsJulianDate : JulianDate;
var
  s : string;
  D, M, Y : integer;
begin
  GetDateAsJulianDate := 0;
  
  if (self.theDate.DateIsValid(self.Text)) then
    begin
      s := Remove_Embeded_Spaces(self.theDate.Text);
      
      MyDate .StrToDMY(s, D, M, Y);
      GetDateAsJulianDate := MyDate .DMYtoDate(D, M, Y);
    end;
end;


procedure Register;
begin
  RegisterComponents('Custom', [TbhChDate]);
end;

end.
